// Programmed by Seung S. yang
// e-mail: syang@cs.kent.edu

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.net.*;
import java.util.Hashtable;

class BitRatePanel extends JPanel {
	
	public BitRatePanel() {
		// set size
		Dimension bitRatePanelDimension = new Dimension(Globals.leftPanelWidth, Globals.leftPanelWindowsHeight);
		setPreferredSize(bitRatePanelDimension);
		setMinimumSize(bitRatePanelDimension);
		setMaximumSize(bitRatePanelDimension);

		// set layout and border
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createEtchedBorder());

		// make rigid area
		add(Box.createRigidArea(new Dimension(Globals.leftPanelWidth, 20)));

		// make label
		JLabel label = new JLabel("Bit rate control");
		label.setFont(new Font("TimesRoman", Font.ITALIC | Font.BOLD, 20));
		label.setAlignmentX(Component.CENTER_ALIGNMENT);
		add(label);

		// make rigid area
		add(Box.createRigidArea(new Dimension(Globals.leftPanelWidth, 10)));

		// make bit rate panel
		add(MakeBitRatePanel());

	}
	public void Show() {
		setVisible(true);
	}
	public void Hide() {
		setVisible(false);
	}

	private JPanel MakeBitRatePanel() {
		JPanel bitRatePanel = new JPanel(false);

		// set size
		Dimension panelDimension = new Dimension(Globals.leftPanelWidth - 20, 200);
		bitRatePanel.setPreferredSize(panelDimension);
		bitRatePanel.setMinimumSize(panelDimension);
		bitRatePanel.setMaximumSize(panelDimension);
		
		// set layout and border
		bitRatePanel.setLayout(new BoxLayout(bitRatePanel, BoxLayout.Y_AXIS));

		// add bit rate slider
		JPanel bitRateSliderPanel = new JPanel();
		Dimension bitRateSliderPanelDimension = new Dimension(Globals.leftPanelWidth - 25, 70);
		bitRateSliderPanel.setPreferredSize(bitRateSliderPanelDimension);
		bitRateSliderPanel.setMinimumSize(bitRateSliderPanelDimension);
		bitRateSliderPanel.setMaximumSize(bitRateSliderPanelDimension);

		bitRateSliderPanel.setLayout(new GridLayout(1,1));
		bitRateSliderPanel.setBorder(BorderFactory.createTitledBorder("Bit rate"));
		
		JSlider bitRateSlider = new JSlider(0, 600, Globals.PFM_INIT);
		Hashtable bitRateLabelTable = new Hashtable();
		bitRateLabelTable.put(new Integer(0), new JLabel("Low (0: Disable)"));
		bitRateLabelTable.put(new Integer(300), new JLabel("Medium"));
		bitRateLabelTable.put(new Integer(600), new JLabel("High"));
		bitRateSlider.setLabelTable(bitRateLabelTable);
		bitRateSlider.setPaintLabels(true);
		bitRateSlider.setMajorTickSpacing(100);
		bitRateSlider.setPaintTicks(true);
		bitRateSlider.addChangeListener(new BitRateChange());

		bitRateSliderPanel.add(bitRateSlider);
		
		bitRatePanel.add(bitRateSliderPanel);

		// add glue
		bitRatePanel.add(Box.createVerticalGlue());

		return bitRatePanel;
	}

	class BitRateChange implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider)e.getSource();
			if (!source.getValueIsAdjusting()) {
				int bitRate = (int)((JSlider)e.getSource()).getValue();
				for (int i = 0; i < Globals.encListModel.getSize(); i++) {
					try {
						Socket ctrlSocket = new Socket((Globals.encListModel.getElementAt(i)).toString(), Globals.CMD_PORT);
						PrintStream ctrlOutputStream = new PrintStream(ctrlSocket.getOutputStream());
						ctrlOutputStream.print("bitrate " + bitRate);
						ctrlOutputStream.flush();
						ctrlSocket.close();
					} catch (UnknownHostException sE) {
						System.err.println(sE);
					} catch (IOException sE) {
						System.err.println(sE);
					}
				}
				// Add status message
				Globals.statusTextArea.append("Bit rate is now " + bitRate + "Kbps\n");
				//System.out.println("Bit rate Value = " + bitRate);
				Toolkit.getDefaultToolkit().beep();
			}
		}
	}

}