// Programmed by Seung S. yang
// e-mail: syang@cs.kent.edu

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.ProgressMonitor;

import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.net.*;
import java.util.StringTokenizer;

class ConfigPanel extends JPanel {
	public ConfigPanel() {
		// set size
		Dimension configPanelDimension = new Dimension(Globals.leftPanelWidth, Globals.leftPanelWindowsHeight);
		setPreferredSize(configPanelDimension);
		setMinimumSize(configPanelDimension);
		setMaximumSize(configPanelDimension);

		// set layout and border
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createEtchedBorder());

		// make rigid area
		add(Box.createRigidArea(new Dimension(Globals.leftPanelWidth, 20)));

		// make label
		JLabel label = new JLabel("Configure");
		label.setFont(new Font("TimesRoman", Font.ITALIC | Font.BOLD, 20));
		label.setAlignmentX(Component.CENTER_ALIGNMENT);
		add(label);

		// make rigid area
		add(Box.createRigidArea(new Dimension(Globals.leftPanelWidth, 10)));

		add(MakeConfigPanel());
	}
	public void Show() {
		setVisible(true);
	}
	public void Hide() {
		setVisible(false);
	}

	private JPanel MakeConfigPanel() {
		JPanel configPanel = new JPanel(false);

		// set size
		Dimension panelDimension = new Dimension(Globals.leftPanelWidth - 20, 200);
		configPanel.setPreferredSize(panelDimension);
		configPanel.setMinimumSize(panelDimension);
		configPanel.setMaximumSize(panelDimension);

		// set layout and border
		configPanel.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		JPanel leftPanel = new JPanel(false);
		JPanel rightPanel = new JPanel(false);
		configPanel.setLayout(new BoxLayout(configPanel, BoxLayout.X_AXIS));

		// Left Panel		
		leftPanel.setLayout(new GridLayout(7,3));
		leftPanel.add(Box.createHorizontalGlue());
		leftPanel.add(new JLabel("Host Name"));
		leftPanel.add(new JLabel("Port Number"));

		leftPanel.add(new JLabel("Control Center"));
		leftPanel.add(Globals.controlCenterName);
		leftPanel.add(Globals.controlCenterPort);
		
		leftPanel.add(new JLabel("Video Server"));
		leftPanel.add(Globals.svrName);
		leftPanel.add(Globals.svrPort);

		leftPanel.add(new JLabel("Decoder"));
		leftPanel.add(Globals.decName);
		leftPanel.add(Globals.decPort);

		leftPanel.add(new JLabel("Multiplexor"));
		leftPanel.add(Globals.muxName);
		leftPanel.add(Globals.muxPort);

		leftPanel.add(new JLabel("Player"));
		leftPanel.add(Globals.playerName);
		leftPanel.add(Globals.playerPort);
		
		JButton distButton = new JButton("Distribute");
		distButton.addActionListener(new Distribute());
		leftPanel.add(distButton);

		JButton hotSwapButton = new JButton("HotSwap");
		hotSwapButton.addActionListener(new HotSwap());
		leftPanel.add(hotSwapButton);

		// Right Panel
		rightPanel.setPreferredSize(new Dimension(Globals.leftPanelWidth/2, Globals.leftPanelWindowsHeight));
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		rightPanel.setLayout(gridbag);
		//c.anchor = GridBagConstraints.NORTH;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		
		JLabel encLabel = new JLabel("Encoders", JLabel.CENTER);
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(encLabel, c);
		rightPanel.add(encLabel);
		
		JPanel encPanel = new JPanel();
		encPanel.setLayout(new BoxLayout(encPanel, BoxLayout.X_AXIS));
		JTextField encName = new JTextField();
		encName.addActionListener(new EncEnter());
		gridbag.setConstraints(encName, c);
		encPanel.add(encName);
		JButton encCancelButton = new JButton("Remove");
		encCancelButton.addActionListener(new EncCancel());
		encPanel.add(encCancelButton);
		//c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridx = 0;
		c.gridy = 1;
		gridbag.setConstraints(encPanel, c);
		rightPanel.add(encPanel);
		
		Globals.encListModel = new DefaultListModel();
		Globals.encList = new JList(Globals.encListModel);
		Globals.encList.setVisibleRowCount(3);
		Globals.encList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane encListScrollPane = new JScrollPane(Globals.encList);
		c.gridx = 0;
		c.gridy = 2;
		c.gridheight = 4;
		c.ipady = 25;
		gridbag.setConstraints(encListScrollPane, c);
		rightPanel.add(encListScrollPane);
		
		JPanel encPortPanel = new JPanel();
		encPortPanel.setLayout(new GridLayout(2,3));
		encPortPanel.add(new JLabel("CmdPort "));
		encPortPanel.add(new JLabel(" DataPort"));
		encPortPanel.add(new JLabel(" QueryPort"));
		encPortPanel.add(Globals.encCmdPort);
		encPortPanel.add(Globals.encDataPort);
		encPortPanel.add(Globals.encQueryPort);

		rightPanel.add(encPortPanel);
		
		configPanel.add(leftPanel);
		configPanel.add(rightPanel);
		
		LoadConfigFile();
		configPanel.updateUI();
		return configPanel;
	}

	private void LoadConfigFile() {
		try {
			BufferedReader confFile = new BufferedReader(new InputStreamReader(new FileInputStream("PXNet.conf")));
			String s;
			StringTokenizer t;

			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			Globals.controlCenterName.setText(t.nextToken());
			Globals.controlCenterPort.setText(t.nextToken());

			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			Globals.svrName.setText(t.nextToken());
			Globals.svrPort.setText(t.nextToken());

			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			Globals.playerName.setText(t.nextToken());
			Globals.playerPort.setText(t.nextToken());

			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			s = t.nextToken();
			Globals.encCmdPort.setText(t.nextToken());
			Globals.encDataPort.setText(t.nextToken());
			Globals.encQueryPort.setText(t.nextToken());

			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");		
			Globals.decName.setText(t.nextToken());
			Globals.decPort.setText(t.nextToken());
			
			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			Globals.muxName.setText(t.nextToken());
			Globals.muxPort.setText(t.nextToken());
			
			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			s = t.nextToken();
			int encNumber;
			encNumber = (new Integer(t.nextToken())).intValue();
			int i;
			for (i = 0; i < encNumber; i++) {
				s = confFile.readLine();
				t = new StringTokenizer(s, " \t\n\r");
				Globals.encListModel.addElement(t.nextToken());
			}
			confFile.close();
		} catch (IOException e) {
			System.err.println(e);
		}
	}

	class HotSwap implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JOptionPane.showMessageDialog(Globals.mainFrame, "Sorry. This feature is not yet implemented.");
		}
	}

	class Distribute implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			int i;
/* for debugging
			System.out.println("Server " + Globals.svrName.getText() + ":" + Globals.svrPort.getText());
			System.out.println("Decoder " + Globals.decName.getText() + ":" + Globals.decPort.getText());
			System.out.println("Multiplexer " + Globals.muxName.getText() + ":" + Globals.muxPort.getText());
			System.out.println("Player " + Globals.playerName.getText() + ":" + Globals.playerPort.getText());
			System.out.println("Encoders ("+ Globals.encListModel.getSize()+")");
			for (i = 0; i < Globals.encListModel.getSize(); i++) {
				System.out.println((Globals.encListModel.getElementAt(i)).toString());
			}
			System.out.println("Encoder Ports " + Globals.encCmdPort.getText() + " : " + Globals.encDataPort.getText() + " : " + Globals.encQueryPort.getText());
*/
			// Add status message
			Globals.statusTextArea.append("Distributing configurations\n");

			// Write configuration file to local disk
			try {
				PrintWriter confFile = new PrintWriter(new BufferedWriter(new FileWriter("PXNet.conf")));
				confFile.println(Globals.controlCenterName.getText() + "\t\t" + Globals.controlCenterPort.getText());
				confFile.println(Globals.svrName.getText() + "\t\t" + Globals.svrPort.getText());
				confFile.println(Globals.playerName.getText() + "\t\t" + Globals.playerPort.getText());
				confFile.println("EncLuncher\t" + Globals.encCmdPort.getText() + "\t" + Globals.encDataPort.getText() + "\t" + Globals.encQueryPort.getText());
				confFile.println(Globals.decName.getText() + "\t\t" + Globals.decPort.getText() + "\t" + String.valueOf((new Integer(Globals.decPort.getText())).intValue() + 1));
				confFile.println(Globals.muxName.getText() + "\t\t" + Globals.muxPort.getText() + "\t" + String.valueOf((new Integer(Globals.muxPort.getText())).intValue() + 1));
				confFile.println("EncLuncherNo" + "\t" + Globals.encListModel.getSize());
				for (i = 0; i < Globals.encListModel.getSize(); i++) {
					confFile.println((Globals.encListModel.getElementAt(i)).toString());
				}
				confFile.close();
			} catch (IOException fE) {
				System.err.println(fE);
			}

			Globals.deployMapPanel.UpdateDeployMapPanel();
			Globals.stressPanel.UpdateStressPanel();
			Globals.leftPanel.validate();
			Globals.rightPanel.validate();

			// send files
			DistributeFiles distributeFiles = new DistributeFiles();
			distributeFiles.start();

			// dixee status
			Globals.dixeeStatusPanel.Update();
		}
	}

	class DistributeFiles extends Thread {
		private int i;
		private int totalWorks;
		private int progress;
		private ProgressMonitor progressMonitor;
		
		public DistributeFiles() {
			ProgressInit();
		}

		public void run() {
			// Add status message
			Globals.statusTextArea.append("Sending configurations to Server\n");
			// Send configuration to server
			SendConfigFile(Globals.svrName.getText());
			if (!ProgressCanceled()) {
				SendSvrFiles(Globals.svrName.getText());
			}
			// Add status message
			Globals.statusTextArea.append("Sending configurations to Decoder\n");
			// Send configuration to decoder
			if (!ProgressCanceled()) {
				SendConfigFile(Globals.decName.getText());
			}
			if (!ProgressCanceled()) {
				SendDecFiles(Globals.decName.getText());
			}
			// Add status message
			Globals.statusTextArea.append("Sending configurations to MUX\n");
			// Send configuration to mux
			if (!ProgressCanceled()) {
				SendConfigFile(Globals.muxName.getText());
			}
			if (!ProgressCanceled()) {
				SendMuxFiles(Globals.muxName.getText());
			}
			// Add status message
			Globals.statusTextArea.append("Sending configurations to Encoders\n");
			// Send configuration to enc
			for (i = 0; i < Globals.encListModel.getSize(); i++) {
				if (!ProgressCanceled()) {
					SendConfigFile((Globals.encListModel.getElementAt(i)).toString());
				}
				if (!ProgressCanceled()) {
					SendEncFiles((Globals.encListModel.getElementAt(i)).toString());
				}
			}
			ProgressClose();
			// Add status message
			Globals.statusTextArea.append("Sending configurations end.\n");

			Toolkit.getDefaultToolkit().beep();
		}
		private void ProgressInit() {
			totalWorks = 3 + 4 + 4 + 8*Globals.encListModel.getSize();
			progressMonitor = new ProgressMonitor(Globals.mainFrame, "Sending files", "initialize connection", 0, totalWorks);
			progressMonitor.setProgress(0);
			progressMonitor.setMillisToDecideToPopup(10);

			progress = 0;
		}

		private void ProgressUpdate() {
			progressMonitor.setProgress(++progress);
			int progressed = (progress * 100)/totalWorks;
			progressMonitor.setNote("Complete " + progressed + "%");
		}
	
		private boolean ProgressCanceled() {
			if (progressMonitor.isCanceled())
				return true;
			else
				return false;
		}
	
		private void ProgressClose() {
			progressMonitor.close();
			progress = 0;
		}

		private void SendConfigFile(String hostName)
		{
			// send configuration to a host
			try {
				Socket ctrlSocket = new Socket(hostName, Globals.CMD_PORT);
				PrintStream ctrlOutputStream = new PrintStream(ctrlSocket.getOutputStream());
				ctrlOutputStream.print("download f xcoder/PXNet.conf");
				ctrlOutputStream.flush();
				ctrlSocket.close();

				Socket dataSocket = new Socket(hostName, Globals.CMD_PORT);
				BufferedOutputStream dataOutputStream = new BufferedOutputStream(dataSocket.getOutputStream());
				BufferedInputStream dataInputStream = new BufferedInputStream( new FileInputStream("PXNet.conf"));
				int c;
				while ((c = dataInputStream.read()) != -1) {
					dataOutputStream.write(c);
				}
				dataOutputStream.flush();
				dataInputStream.close();
				dataSocket.close();
			} catch (UnknownHostException sE) {
				System.err.println(sE);
			} catch (IOException sE) {
				System.err.println(sE);
			}

//			try {Thread.sleep(100);} catch (InterruptedException e) {};
			// update progress monitor
			ProgressUpdate();
		}

		private void SendCmd(String hostName, String cmd)
		{
			// send command to a host
			try {
				Socket ctrlSocket = new Socket(hostName, Globals.CMD_PORT);
				PrintStream ctrlOutputStream = new PrintStream(ctrlSocket.getOutputStream());
				ctrlOutputStream.print(cmd);
				ctrlOutputStream.flush();
				ctrlSocket.close();
			} catch (UnknownHostException sE) {
				System.err.println(sE);
			} catch (IOException sE) {
				System.err.println(sE);
			}

		}
	
		private void SendFile(String hostName, String fileName)
		{
			// send a file to a host
			try {
				Socket dataSocket = new Socket(hostName, Globals.CMD_PORT);
				BufferedOutputStream dataOutputStream = new BufferedOutputStream(dataSocket.getOutputStream());
				BufferedInputStream dataInputStream = new BufferedInputStream(new FileInputStream(fileName));
				int c;
				while ((c = dataInputStream.read()) != -1) {
					dataOutputStream.write(c);
				}
				dataOutputStream.flush();
				dataInputStream.close();
				dataSocket.close();
			} catch (UnknownHostException sE) {
				System.err.println(sE);
			} catch (IOException sE) {
				System.err.println(sE);
			}

//			try {Thread.sleep(100);} catch (InterruptedException e) {};
			// update progress monitor
			ProgressUpdate();
		}
	
		private void SendSvrFiles(String hostName)
		{
			// Send Svr to host
			SendCmd(hostName, "download x xcoder/svr/svr");
			SendFile(hostName, "xcoder/svr/svr");
			SendCmd(hostName, "download f xcoder/svr/svr.m2v");
			SendFile(hostName, "xcoder/svr/svr.m2v");
		}
	
		private void SendDecFiles(String hostName)
		{
			// Send Dec to host
			SendCmd(hostName, "download x xcoder/xdec/mpeg2decode");
			SendFile(hostName, "xcoder/xdec/mpeg2decode");
			SendCmd(hostName, "download x xcoder/xdec/cleanfiles");
			SendFile(hostName, "xcoder/xdec/cleanfiles");
			SendCmd(hostName, "download f xcoder/xdec/MnN.par");
			SendFile(hostName, "xcoder/xdec/MnN.par");
		}

		private void SendEncFiles(String hostName)
		{
			// Send Enc to host
			SendCmd(hostName, "download x xcoder/xenc/mpeg2encode");
			SendFile(hostName, "xcoder/xenc/mpeg2encode");
			SendCmd(hostName, "download x xcoder/xenc/xlun");
			SendFile(hostName, "xcoder/xenc/xlun");
			SendCmd(hostName, "download x xcoder/xenc/cleanfiles");
			SendFile(hostName, "xcoder/xenc/cleanfiles");
			SendCmd(hostName, "download f xcoder/xenc/adjmquant");
			SendFile(hostName, "xcoder/xenc/adjmquant");
			SendCmd(hostName, "download f xcoder/xenc/adjustq");
			SendFile(hostName, "xcoder/xenc/adjustq");
			SendCmd(hostName, "download f xcoder/xenc/rate.par");
			SendFile(hostName, "xcoder/xenc/rate.par");
			SendCmd(hostName, "download f xcoder/xenc/MnN.par");
			SendFile(hostName, "xcoder/xenc/MnN.par");
		}

		private void SendMuxFiles(String hostName)
		{
			// Send Mux to host
			SendCmd(hostName, "download x xcoder/xmux/xmux");
			SendFile(hostName, "xcoder/xmux/xmux");
			SendCmd(hostName, "download x xcoder/xmux/cleanfiles");
			SendFile(hostName, "xcoder/xmux/cleanfiles");
			SendCmd(hostName, "download f xcoder/xmux/gopsize");
			SendFile(hostName, "xcoder/xmux/gopsize");
			SendCmd(hostName, "download x xcoder/xmux/synker");
			SendFile(hostName, "xcoder/xmux/synker");
		}
	}
	

	class EncEnter implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JTextField txtField = (JTextField)e.getSource();
			String text = txtField.getText();
			txtField.setText("");
			Toolkit.getDefaultToolkit().beep();
			Globals.encListModel.addElement(text);
		}
	}
	
	class EncCancel implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			int index = Globals.encList.getSelectedIndex();
			Globals.encListModel.remove(index);
		}
	}

}
