// Programmed by Seung S. Yang
// e-mail: syang@cs.kent.edu

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.Timer;

import java.awt.*;
import java.awt.event.*;

import java.io.*;

class ControlCenter {
	public static void main(String[] args) {
		// set screen width & height
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Globals.mainWindowWidth = toolkit.getScreenSize().width;
		Globals.mainWindowHeight = toolkit.getScreenSize().height;

		// set left panel windows height
		Globals.leftPanelWindowsHeight = Globals.mainWindowHeight /2;
		
		// set right panel windows size
		Globals.rightPanelWidth = Globals.mainWindowWidth - Globals.titlePanelWidth - Globals.leftPanelWidth;
		Globals.rightPanelWindowHeight = Globals.mainWindowHeight /2;

		// show splash window
		Globals.splash = new Splash();
		Globals.splash.Hide();

		// make main frame
		Globals.mainFrame = new JFrame("Kent State University VSM ControlCenter: SONET");
		Globals.mainFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {System.exit(0);}
		});
		Globals.mainFrame.getContentPane().setLayout(new BoxLayout(Globals.mainFrame.getContentPane(), BoxLayout.X_AXIS));

		// make menu
		JMenuBar menuBar = new JMenuBar();
		MakeMenu(menuBar);
		Globals.mainFrame.setJMenuBar(menuBar);

		// make main panels

		JPanel leftTitlePanel = new JPanel();
		leftTitlePanel.setBackground(Color.blue);
		leftTitlePanel.setLayout(new BoxLayout(leftTitlePanel, BoxLayout.Y_AXIS));
		JLabel leftTitleLabel = new JLabel(new ImageIcon("images/LeftTitle.gif"));
		leftTitlePanel.add(Box.createVerticalGlue());
		leftTitlePanel.add(leftTitleLabel);

		Globals.leftPanel = new LeftPanel();
		Globals.rightPanel = new RightPanel();

		Globals.mainFrame.getContentPane().add(leftTitlePanel);
		Globals.mainFrame.getContentPane().add(Globals.leftPanel);
		Globals.mainFrame.getContentPane().add(Globals.rightPanel);

		Globals.mainFrame.setSize(Globals.mainWindowWidth, Globals.mainWindowHeight);
		Globals.mainFrame.setVisible(true);
	}

	public static void MakeMenu(JMenuBar menuBar) {
		// make file menu
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');

		// make menu item in file menu
		JMenuItem menuItem = new JMenuItem("Exit", KeyEvent.VK_X);
		menuItem.addActionListener(new Exit());
		fileMenu.add(menuItem);

		menuBar.add(fileMenu);

		// make view menu
		JMenu viewMenu = new JMenu("View");
		viewMenu.setMnemonic('V');
		
		// make menu item in view menu
		menuItem = new JMenuItem("Protocols", KeyEvent.VK_P);
		menuItem.addActionListener(new Protocols());
		viewMenu.add(menuItem);
		
		menuBar.add(viewMenu);

		// make help menu
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic('H');

		// make menuitem in help menu
		menuItem = new JMenuItem("Connect Medianet web site", KeyEvent.VK_M);
		menuItem.addActionListener(new MedianetWeb());
		helpMenu.add(menuItem);

		helpMenu.addSeparator();

		menuItem = new JMenuItem("About", KeyEvent.VK_A);
		menuItem.addActionListener(new About());
		helpMenu.add(menuItem);

		menuBar.add(helpMenu);

		// make space
		menuBar.add(Box.createHorizontalGlue());
		
		// make KSU logo
		menuBar.add(new JLabel(new ImageIcon("images/ksulogo.gif")));

	}
}

class MedianetWeb implements ActionListener {
	public void actionPerformed(ActionEvent e) {
		String link = "http://www.medianet.kent.edu";
		String cmd = "cmd /c start /b "+ link;
		try {
			Runtime.getRuntime().exec(cmd);
		} catch (IOException eRun) {
			System.err.println(eRun);
		}
	}
}

class About implements ActionListener {
	public void actionPerformed(ActionEvent e) {
		Globals.splash.Show();
	}
}

class Protocols implements ActionListener {
	private static final int PROTOCOLS_ANIMATION_DELAY = 300;
	private ImageIcon protocolsDImageIcons[] = new ImageIcon[10];
	private ImageIcon protocolsEImageIcons[] = new ImageIcon[6];
	private ImageIcon protocolsMImageIcons[] = new ImageIcon[8];
	private JLabel protocolsDLabel;
	private JLabel protocolsELabel;
	private JLabel protocolsMLabel;
	private Timer timer;

	static int dIconNo = 0;
	static int eIconNo = 0;
	static int mIconNo = 0;
	static boolean eActive = false;
	static boolean mActive = false;
	public void actionPerformed(ActionEvent e) {
		int i;
		if (Globals.protocolsFrame == null) {
			// make protocols frame
			Globals.protocolsFrame = new JFrame("Protocols");
			Globals.protocolsFrame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {timer.stop(); Globals.protocolsFrame.setVisible(false);}
			});
			Globals.protocolsFrame.getContentPane().setLayout(new BoxLayout(Globals.protocolsFrame.getContentPane(), BoxLayout.X_AXIS));

			// animation panel
			for (i = 0; i <= 9; i++) {
				protocolsDImageIcons[i] = new ImageIcon("images/protocolsD"+i+".gif");
			}
			for (i = 0; i <= 5; i++) {
				protocolsEImageIcons[i] = new ImageIcon("images/protocolsE"+i+".gif");
			}
			for (i = 0; i <= 7; i++) {
				protocolsMImageIcons[i] = new ImageIcon("images/protocolsM"+i+".gif");
			}
			protocolsDLabel = new JLabel("X-Dec", protocolsDImageIcons[0], JLabel.CENTER);
			protocolsDLabel.setVerticalTextPosition(JLabel.BOTTOM);
			protocolsDLabel.setHorizontalTextPosition(JLabel.CENTER);

			protocolsELabel = new JLabel("X-Enc", protocolsEImageIcons[0], JLabel.CENTER);
			protocolsELabel.setVerticalTextPosition(JLabel.BOTTOM);
			protocolsELabel.setHorizontalTextPosition(JLabel.CENTER);

			protocolsMLabel = new JLabel("X-Mux", protocolsMImageIcons[0], JLabel.CENTER);
			protocolsMLabel.setVerticalTextPosition(JLabel.BOTTOM);
			protocolsMLabel.setHorizontalTextPosition(JLabel.CENTER);

			Globals.protocolsFrame.getContentPane().add(protocolsDLabel);
			Globals.protocolsFrame.getContentPane().add(protocolsELabel);
			Globals.protocolsFrame.getContentPane().add(protocolsMLabel);

			// set protocols frame width & height
			Globals.protocolsFrame.setSize(Globals.mainWindowWidth, Globals.mainWindowHeight);

		}
		Globals.protocolsFrame.setVisible(true);
		timer = new Timer(PROTOCOLS_ANIMATION_DELAY, new ProtocolsAction());
		timer.start();
	}

	class ProtocolsAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			dIconNo = ++dIconNo % 10;
			protocolsDLabel.setIcon(protocolsDImageIcons[dIconNo]);
			if (eActive) {
				eIconNo = ++eIconNo % 6;
			}
			protocolsELabel.setIcon(protocolsEImageIcons[eIconNo]);
			if (mActive) {
				mIconNo = ++mIconNo % 8;
			}
			protocolsMLabel.setIcon(protocolsMImageIcons[mIconNo]);
			if (mIconNo == 7) {
				mIconNo = 0;
				mActive = false;
			}
			if (eIconNo == 5) {
				eIconNo = 0;
				eActive = false;
				mActive = true;
			}
			if (dIconNo == 9) {
				dIconNo = 1;
				eActive = true;
			}
			timer.setDelay(PROTOCOLS_ANIMATION_DELAY);
			timer.setInitialDelay(PROTOCOLS_ANIMATION_DELAY * 2);
			timer.restart();
		}
	}
}

class Exit implements ActionListener {
	public void actionPerformed(ActionEvent e) {
		System.exit(0);
	}
}
