// Programmed by Seung S. yang
// e-mail: syang@cs.kent.edu

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

import java.awt.*;

class DeployMapPanel extends JPanel {
	private JPanel deployMapPanel;
	public DeployMapPanel() {
		// set size
		Dimension deployMapPanelDimension = new Dimension(Globals.rightPanelWidth - 2*Globals.marginWidth, Globals.rightPanelWindowHeight);
		setPreferredSize(deployMapPanelDimension);
		setMinimumSize(deployMapPanelDimension);
		setMaximumSize(deployMapPanelDimension);

		// set layout and border
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createEtchedBorder());

		// make rigid area
		add(Box.createRigidArea(new Dimension(Globals.rightPanelWidth - 2*Globals.marginWidth, 30)));

		// make label
		JLabel label = new JLabel("Application Deployment Map");
		label.setFont(new Font("TimesRoman", Font.ITALIC | Font.BOLD, 20));
		label.setAlignmentX(Component.CENTER_ALIGNMENT);
		add(label);

		// make rigid area
		add(Box.createRigidArea(new Dimension(Globals.rightPanelWidth - 2*Globals.marginWidth, 50)));

		// make deploy map
		MakeDeployMapPanel();
		add(deployMapPanel);
	}
	public void Show() {
		setVisible(true);
	}
	public void Hide() {
		setVisible(false);
	}

	private void MakeDeployMapPanel() {
		// deploy map Panel
		deployMapPanel = new JPanel(false);

		// set layout and border
		deployMapPanel.setLayout(new BoxLayout(deployMapPanel, BoxLayout.X_AXIS));

		deployMapPanel.add(Box.createHorizontalGlue());
		deployMapPanel.add(GetServerDiagram());
		deployMapPanel.add(GetDecoderDiagram());
		deployMapPanel.add(GetEncoderDiagram());
		deployMapPanel.add(GetMuxDiagram());
		deployMapPanel.add(GetPlayerDiagram());
		deployMapPanel.add(Box.createHorizontalGlue());
	}

	protected void UpdateDeployMapPanel() {
		deployMapPanel.removeAll();
		deployMapPanel.add(Box.createHorizontalGlue());
		deployMapPanel.add(GetServerDiagram());
		deployMapPanel.add(GetDecoderDiagram());
		deployMapPanel.add(GetEncoderDiagram());
		deployMapPanel.add(GetMuxDiagram());
		deployMapPanel.add(GetPlayerDiagram());
		deployMapPanel.add(Box.createHorizontalGlue());
		deployMapPanel.revalidate();
		deployMapPanel.update(deployMapPanel.getGraphics());
	}
	
	private Component GetServerDiagram() {
		JPanel panel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		String svrNameString = new String(Globals.svrName.getText());
		if (svrNameString == "")
			return panel;


		panel.setBorder(BorderFactory.createTitledBorder(svrNameString));
		panel.add(new JLabel(new ImageIcon("images/server.gif"), JLabel.CENTER));

		if (svrNameString.compareTo(Globals.decName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("images/decoder.gif"), JLabel.CENTER));
		}
		
		if (svrNameString.compareTo(Globals.muxName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("images/mux.gif"), JLabel.CENTER));
		}
		
		if (svrNameString.compareTo(Globals.playerName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("images/player.gif"), JLabel.CENTER));
		}

		for (int i = 0; i < Globals.encListModel.getSize(); i++) {
			if (svrNameString.compareTo((Globals.encListModel.getElementAt(i)).toString()) == 0) {
				panel.add(new JLabel(new ImageIcon("images/encoder.gif"), JLabel.CENTER));
			}
		}
		
		return panel;
	}
	
	private Component GetDecoderDiagram() {
		JPanel panel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		String decNameString = new String(Globals.decName.getText());
		if (decNameString == "")
			return panel;

		if (decNameString.compareTo(Globals.svrName.getText()) == 0)
			return panel;

		deployMapPanel.add(new JLabel(new ImageIcon("images/connection.gif"), JLabel.CENTER));
		panel.setBorder(BorderFactory.createTitledBorder(decNameString));
		panel.add(new JLabel(new ImageIcon("images/decoder.gif"), JLabel.CENTER));

		if (decNameString.compareTo(Globals.muxName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("images/mux.gif"), JLabel.CENTER));
		}
		
		if (decNameString.compareTo(Globals.playerName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("images/player.gif"), JLabel.CENTER));
		}

		for (int i = 0; i < Globals.encListModel.getSize(); i++) {
			if (decNameString.compareTo((Globals.encListModel.getElementAt(i)).toString()) == 0) {
				panel.add(new JLabel(new ImageIcon("images/encoder.gif"), JLabel.CENTER));
			}
		}

		return panel;
	}

	private Component GetEncoderDiagram() {
		JPanel panel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

		deployMapPanel.add(new JLabel(new ImageIcon("images/connection.gif"), JLabel.CENTER));

		for (int i=0; i < Globals.encListModel.getSize(); i++) {
			JPanel subPanel = new JPanel(false);
			subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.Y_AXIS));
			String encNameString = new String(Globals.encListModel.getElementAt(i).toString());
			if (encNameString == "") {
				panel.add(subPanel);
				continue;
			}

			if (encNameString.compareTo(Globals.svrName.getText()) == 0) {
				panel.add(subPanel);
				continue;
			}

			if (encNameString.compareTo(Globals.decName.getText()) == 0) {
				panel.add(subPanel);
				continue;
			}

			subPanel.setBorder(BorderFactory.createTitledBorder(encNameString));
			subPanel.add(new JLabel(new ImageIcon("images/encoder.gif"), JLabel.CENTER));

			if (encNameString.compareTo(Globals.muxName.getText()) == 0) {
				subPanel.add(new JLabel(new ImageIcon("images/mux.gif"), JLabel.CENTER));
			}
		
			if (encNameString.compareTo(Globals.playerName.getText()) == 0) {
				subPanel.add(new JLabel(new ImageIcon("images/player.gif"), JLabel.CENTER));
			}
			panel.add(subPanel);
		}
		return panel;
	}

	private Component GetMuxDiagram() {
		JPanel panel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		String muxNameString = new String(Globals.muxName.getText());
		if (muxNameString == "")
			return panel;

		if (muxNameString.compareTo(Globals.svrName.getText()) == 0)
			return panel;
			
		if (muxNameString.compareTo(Globals.decName.getText()) == 0)
			return panel;

		for (int i = 0; i < Globals.encListModel.getSize(); i++) {
			if (muxNameString.compareTo((Globals.encListModel.getElementAt(i)).toString()) == 0) {
				return panel;
			}
		}

		deployMapPanel.add(new JLabel(new ImageIcon("images/connection.gif"), JLabel.CENTER));		
		panel.setBorder(BorderFactory.createTitledBorder(muxNameString));
		panel.add(new JLabel(new ImageIcon("images/mux.gif"), JLabel.CENTER));

		if (muxNameString.compareTo(Globals.playerName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("images/player.gif"), JLabel.CENTER));
		}

		return panel;
	}

	private Component GetPlayerDiagram() {
		JPanel panel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		String playerNameString = new String(Globals.playerName.getText());
		if (playerNameString == "")
			return panel;

		if (playerNameString.compareTo(Globals.svrName.getText()) == 0)
			return panel;
			
		if (playerNameString.compareTo(Globals.decName.getText()) == 0)
			return panel;

		for (int i = 0; i < Globals.encListModel.getSize(); i++) {
			if (playerNameString.compareTo((Globals.encListModel.getElementAt(i)).toString()) == 0) {
				return panel;
			}
		}

		deployMapPanel.add(new JLabel(new ImageIcon("images/connection.gif"), JLabel.CENTER));		
		panel.setBorder(BorderFactory.createTitledBorder(playerNameString));
		panel.add(new JLabel(new ImageIcon("images/player.gif"), JLabel.CENTER));

		return panel;
	}
}