// Programmed by Seung S. yang
// e-mail: syang@cs.kent.edu

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.net.*;

class NavNStatusPanel extends JPanel {
	private boolean startFlag = true;
	private ImageIcon navStart[] = new ImageIcon[2];
	private ImageIcon navStop[] = new ImageIcon[2];
	private ImageIcon navConfigure[] = new ImageIcon[2];
	private ImageIcon navRun[] = new ImageIcon[2];
	private ImageIcon navStress[] = new ImageIcon[2];
	private ImageIcon navBitrate[] = new ImageIcon[2];
	public NavNStatusPanel() {
		// set size
		Dimension navNStatusPanelDimension = new Dimension(Globals.leftPanelWidth, Globals.navNStatusPanelHeight);
		setPreferredSize(navNStatusPanelDimension);
		setMinimumSize(navNStatusPanelDimension);
		setMaximumSize(navNStatusPanelDimension);

		// set layout and border
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		setBorder(BorderFactory.createEtchedBorder());

		// load nav images
		LoadNavImages();

		// make nav label
		Globals.navLabel = new JLabel(navStart[0]);
		add(Globals.navLabel);
		Globals.navLabel.addMouseMotionListener(new MouseMove());
		Globals.navLabel.addMouseListener(new MouseOn());

		// make horizontal glue
		add(Box.createHorizontalGlue());
		
		// make status text area
		Globals.statusTextArea = new JTextArea("ready\n");
		Globals.statusTextArea.setForeground(Color.gray);
		Globals.statusTextArea.setBackground(new Color(206,207,206));
		JScrollPane statusScrollPane = new JScrollPane(Globals.statusTextArea);
		statusScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		statusScrollPane.setPreferredSize(new Dimension(Globals.leftPanelWidth - Globals.navNStatusPanelWidth, Globals.navNStatusPanelHeight));
		statusScrollPane.setMinimumSize(new Dimension(Globals.leftPanelWidth - Globals.navNStatusPanelWidth, Globals.navNStatusPanelHeight));
		statusScrollPane.setMaximumSize(new Dimension(Globals.leftPanelWidth - Globals.navNStatusPanelWidth, Globals.navNStatusPanelHeight));
		add(statusScrollPane);
	}
	
	private void LoadNavImages() {
		navStart[0] = new ImageIcon("images/NavStart.gif");
		navStart[1] = new ImageIcon("images/NavStart_h.gif");
		navStop[0] = new ImageIcon("images/NavStop.gif");
		navStop[1] = new ImageIcon("images/NavStop_h.gif");
		navConfigure[0] = new ImageIcon("images/NavStart_configure_h.gif");
		navConfigure[1] = new ImageIcon("images/NavStop_configure_h.gif");
		navRun[0] = new ImageIcon("images/NavStart_run_h.gif");
		navRun[1] = new ImageIcon("images/navStop_run_h.gif");
		navStress[0] = new ImageIcon("images/NavStart_stress_h.gif");
		navStress[1] = new ImageIcon("images/NavStop_stress_h.gif");
		navBitrate[0] = new ImageIcon("images/NavStart_bitrate_h.gif");
		navBitrate[1] = new ImageIcon("images/NavStop_bitrate_h.gif");
	}

	private boolean InCircle(Point s, Point c, int r) {
		if ((int)Math.sqrt(Math.abs(s.x - c.x) * Math.abs(s.x - c.x) + Math.abs(s.y - c.y) * Math.abs(s.y - c.y)) <= r)
			return true;
		else
			return false;
	}

	class MouseMove implements MouseMotionListener {
		public void mouseMoved(MouseEvent e) {
			Point p = e.getPoint();
			//System.out.println("mouse moved x = "+e.getX()+ " y = " + e.getY());
			if (InCircle(p, new Point(navStart[0].getIconWidth()/2, navStart[0].getIconHeight()/2), 36)) {
				if (startFlag) {
					Globals.navLabel.setIcon(navStart[1]);
				} else {
					Globals.navLabel.setIcon(navStop[1]);
				}
			} else	if (InCircle(p, new Point(navStart[0].getIconWidth()/2, navStart[0].getIconHeight()/2), 70)) {
				if (p.x < (navStart[0].getIconWidth()/2)) {
					if (p.y < (navStart[0].getIconHeight()/2)) {
						// nav configure
						if (startFlag) {
							Globals.navLabel.setIcon(navConfigure[0]);
						} else {
							Globals.navLabel.setIcon(navConfigure[1]);
						}
					} else {
						// nav bitrate
						if (startFlag) {
							Globals.navLabel.setIcon(navBitrate[0]);
						} else {
							Globals.navLabel.setIcon(navBitrate[1]);
						}
					}
				} else {
					if (p.y < (navStart[0].getIconHeight()/2)) {
						// nav run
						if (startFlag) {
							Globals.navLabel.setIcon(navRun[0]);
						} else {
							Globals.navLabel.setIcon(navRun[1]);
						}
					} else {
						// nav stress
						if (startFlag) {
							Globals.navLabel.setIcon(navStress[0]);
						} else {
							Globals.navLabel.setIcon(navStress[1]);
						}
					}
				}
			}
		}
		public void mouseDragged(MouseEvent e) {
			//System.out.println("mouse dragged x = "+e.getX()+ " y = " + e.getY());
		}
	}
	class MouseOn implements MouseListener {
		public void mouseClicked(MouseEvent e) {
			Point p = e.getPoint();
			if (InCircle(p, new Point(navStart[0].getIconWidth()/2, navStart[0].getIconHeight()/2), 36)) {
				if (startFlag) {
					Start();
					startFlag = false;
					Globals.navLabel.setIcon(navStop[1]);
				} else {
					Stop();
					startFlag = true;
					Globals.navLabel.setIcon(navStart[1]);
				}
			} else	if (InCircle(p, new Point(navStart[0].getIconWidth()/2, navStart[0].getIconHeight()/2), 70)) {
				if (p.x < (navStart[0].getIconWidth()/2)) {
					if (p.y < (navStart[0].getIconHeight()/2)) {
						// nav configure
						Globals.runPanel.Hide();
						Globals.bitRatePanel.Hide();
						Globals.stressPanel.Hide();
						Globals.configPanel.Show();
					} else {
						// nav bitrate
						Globals.runPanel.Hide();
						Globals.configPanel.Hide();
						Globals.stressPanel.Hide();
						Globals.bitRatePanel.Show();
					}
				} else {
					if (p.y < (navStart[0].getIconHeight()/2)) {
						// nav run
						Globals.configPanel.Hide();
						Globals.bitRatePanel.Hide();
						Globals.stressPanel.Hide();
						Globals.runPanel.Show();
					} else {
						// nav stress
						Globals.runPanel.Hide();
						Globals.configPanel.Hide();
						Globals.bitRatePanel.Hide();
						Globals.stressPanel.UpdateStressPanel();
						Globals.stressPanel.Show();
					}
				}
			}
		}
		public void mouseEntered(MouseEvent e) {
		}
		public void mousePressed(MouseEvent e) {
		}
		public void mouseReleased(MouseEvent e) {
		}
		public void mouseExited(MouseEvent e) {
			if (startFlag) {
				Globals.navLabel.setIcon(navStart[0]);
			} else {
				Globals.navLabel.setIcon(navStop[0]);
			}
		}
	}

	private void Start() {
		// Add status message
		Globals.statusTextArea.append("Start transcoding\n");

		// Server start
		SendMessage(Globals.svrName.getText(), "start svr");
		// Encoders start
		int i;
		for (i = 0; i < Globals.encListModel.getSize(); i++) {
			SendMessage((Globals.encListModel.getElementAt(i)).toString(), "start enc");
		}
		// Mux start
		SendMessage(Globals.muxName.getText(), "start mux");
		// Synker start
		SendMessage(Globals.muxName.getText(), "start synker");
		// Decoder start
		SendMessage(Globals.decName.getText(), "start dec");
		// Player
		String cmd = "cmd /c start /b "+ "XPlayer.exe";
/* For running player seperate machine
		try {
			Runtime.getRuntime().exec(cmd);
		} catch (IOException eRun) {
			System.err.println(eRun);
		}
*/
		Toolkit.getDefaultToolkit().beep();
	}

	private void Stop() {
		// Server stop
		SendMessage(Globals.svrName.getText(), "stop svr");
		// Encoders stop
		int i;
		for (i = 0; i < Globals.encListModel.getSize(); i++) {
			SendMessage((Globals.encListModel.getElementAt(i)).toString(), "stop enc");
		}
		// Mux stop
		SendMessage(Globals.muxName.getText(), "stop mux");
		// Synker stop
		SendMessage(Globals.muxName.getText(), "stop synker");
		// Decoder stop
		SendMessage(Globals.decName.getText(), "stop dec");
		// Add status message
		Globals.statusTextArea.append("Stop transcoding\n");
		Toolkit.getDefaultToolkit().beep();
	}

	private void SendMessage(String hostName, String message)
	{
		try {
			Socket hostCtrlSocket = new Socket(hostName, Globals.CMD_PORT);
			PrintStream hostCtrlOutputStream = new PrintStream(hostCtrlSocket.getOutputStream());
			hostCtrlOutputStream.print(message);
			hostCtrlOutputStream.flush();
			hostCtrlSocket.close();
		} catch (UnknownHostException sE) {
			System.err.println(sE);
		} catch (IOException sE) {
			System.err.println(sE);
		}
	}
}


