// Programmed by Seung S. yang
// e-mail: syang@cs.kent.edu

import javax.swing.border.TitledBorder;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.Timer;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.net.*;
import java.util.Hashtable;
import java.util.StringTokenizer;

class RunPanel extends JPanel {

	private JSlider fpsSlider = new JSlider(0, Globals.MAX_FPS, FPS_INIT);

	private static final int DELAY_FACTOR = 300;
	private static final int FPS_INIT = 0;
	private ImageIcon mLabImageIcons[] = new ImageIcon[19];
	private JLabel mLabLabel;
	private int delay = DELAY_FACTOR;
	private int iconNo = 0;
	private Timer timer;

	public RunPanel() {
		// set size
		Dimension runPanelDimension = new Dimension(Globals.leftPanelWidth, Globals.leftPanelWindowsHeight);
		setPreferredSize(runPanelDimension);
		setMinimumSize(runPanelDimension);
		setMaximumSize(runPanelDimension);

		// set layout and border
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createEtchedBorder());

		// make rigid area
		add(Box.createRigidArea(new Dimension(Globals.leftPanelWidth, 20)));

		// make label
		JLabel label = new JLabel("Run");
		label.setFont(new Font("TimesRoman", Font.ITALIC | Font.BOLD, 20));
		label.setAlignmentX(Component.CENTER_ALIGNMENT);
		add(label);

		// make rigid area
		add(Box.createRigidArea(new Dimension(Globals.leftPanelWidth, 10)));

		add(MakeRunPanel());
	}
	public void Show() {
		setVisible(true);
	}
	public void Hide() {
		setVisible(false);
	}

	private JPanel MakeRunPanel() {
		JPanel runPanel = new JPanel();
		// set size
		Dimension panelDimension = new Dimension(Globals.leftPanelWidth - 20, 200);
		runPanel.setPreferredSize(panelDimension);
		runPanel.setMinimumSize(panelDimension);
		runPanel.setMaximumSize(panelDimension);

		// set layout and border
		runPanel.setLayout(new BoxLayout(runPanel, BoxLayout.Y_AXIS));
		
		// performance panel
		JPanel perfPanel = new JPanel();
		perfPanel.setBorder(BorderFactory.createTitledBorder("Performance meter (%)"));
		perfPanel.setLayout(new GridLayout(1,1));
		Dimension perfPanelDimension = new Dimension(Globals.leftPanelWidth - 25, 80);
		perfPanel.setPreferredSize(perfPanelDimension);
		perfPanel.setMinimumSize(perfPanelDimension);
		perfPanel.setMaximumSize(perfPanelDimension);
		JSlider perfSlider = new JSlider(1, 100, Globals.PFM_INIT);
		Hashtable perfLabelTable = new Hashtable();
		perfLabelTable.put(new Integer(1), new JLabel("Low"));
		perfLabelTable.put(new Integer(50), new JLabel("Medium"));
		perfLabelTable.put(new Integer(100), new JLabel("High"));
		perfSlider.setLabelTable(perfLabelTable);
		perfSlider.setPaintLabels(true);
		perfSlider.setMajorTickSpacing(10);
		perfSlider.setPaintTicks(true);
		perfSlider.addChangeListener(new PerfChange());
		perfPanel.add(perfSlider);
		
		// fps panel
		JPanel fpsPanel = new JPanel();
		fpsPanel.setLayout(new BoxLayout(fpsPanel, BoxLayout.X_AXIS));
		Dimension fpsPanelDimension = new Dimension(Globals.leftPanelWidth - 25, 80);
		fpsPanel.setPreferredSize(fpsPanelDimension);
		fpsPanel.setMinimumSize(fpsPanelDimension);
		fpsPanel.setMaximumSize(fpsPanelDimension);
		JPanel fpsSliderPanel = new JPanel();
		TitledBorder fpsSliderTitledBorder = BorderFactory.createTitledBorder("Current frame rates (fps)");
		fpsSliderTitledBorder.setTitleColor(Color.gray);
		fpsSliderPanel.setBorder(fpsSliderTitledBorder);
		fpsSliderPanel.setLayout(new GridLayout(1,1));
		fpsSlider.setMajorTickSpacing(10);
		fpsSlider.setMinorTickSpacing(1);
		fpsSlider.setPaintTicks(true);
		fpsSlider.setPaintLabels(true);
		fpsSliderPanel.add(fpsSlider);
		fpsPanel.add(fpsSliderPanel);
		
		FpsSliderUpdater fpsSliderHandler = new FpsSliderUpdater();
		fpsSliderHandler.start();
		
		// animation panel
		for (int i = 1; i <= 19; i++) {
			mLabImageIcons[i-1] = new ImageIcon("images/m"+i+".gif");
		}
		mLabLabel = new JLabel(mLabImageIcons[0], JLabel.CENTER);
		mLabLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		fpsPanel.add(mLabLabel);
		timer = new Timer(delay, new MLabAction());
		
		startMLab();
		
		runPanel.add(perfPanel);
		runPanel.add(fpsPanel);
		
		// add glue
		runPanel.add(Box.createVerticalGlue());
		
		return runPanel;
	}
	
	private void startMLab() {
		timer.start();
	}
	
	private void stopMLab() {
		timer.stop();
	}

	class PerfChange implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider)e.getSource();
			if (!source.getValueIsAdjusting()) {
				if (Globals.decName.getText() != "") {
					int perf = (int)((JSlider)e.getSource()).getValue();
					try {
						Socket decSocket = new Socket(Globals.decName.getText(), (new Integer(Globals.decPort.getText())).intValue());
						PrintStream decOutputStream = new PrintStream(decSocket.getOutputStream());
						decOutputStream.print(perf);
						decOutputStream.flush();
						decSocket.close();
					} catch (UnknownHostException sE) {
						System.err.println(sE);
					} catch (IOException sE) {
						System.err.println(sE);
					}
					// Add status message
					Globals.statusTextArea.append("Performance Value is now " + perf + "\n");
					//System.out.println("Perf Value = " + perf);
				}
				Toolkit.getDefaultToolkit().beep();
			}
		}
	}

	class FpsSliderUpdater extends Thread {
		private ServerSocket ss;
		public FpsSliderUpdater() {
			try {
				ss = new ServerSocket(Globals.FRAME_SLIDER_PORT);
				//ss = new ServerSocket((new Integer(Globals.controlCenterPort.getText())).intValue());
			} catch (IOException e) {
				System.err.println(e);
			}
		}
		public void run() {
			while (true) {
				try {
					Socket s = ss.accept();
					BufferedReader fss = new BufferedReader(new InputStreamReader(s.getInputStream()));
					String str = fss.readLine();
					s.close();
					StringTokenizer t = new StringTokenizer(str, " \t\n\r");
					String fps = new String(t.nextToken());
					int value = 0;
					try {
						value = (new Integer(fps)).intValue();
					} catch (NumberFormatException nE) {
						System.err.println(nE);
					}
					fpsSlider.setValue(value);
					//System.out.println("Get FPS = " + value);
				} catch (IOException e) {
					System.err.println(e);
				}
			}
		}
	}

	class MLabAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			int sliderValue = fpsSlider.getValue();
			if (sliderValue != 0) {
				iconNo = ++iconNo % 19;
				mLabLabel.setIcon(mLabImageIcons[iconNo]);
				delay = DELAY_FACTOR/sliderValue;
				timer.setDelay(delay);
				timer.setInitialDelay(delay * 2);
				timer.restart();
			}
		}
	}

}
