// Programmed by Seung S. yang
// e-mail: syang@cs.kent.edu

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.*;

import java.awt.*;

import java.io.*;
import java.net.*;
import java.util.Hashtable;

class StressPanel extends JPanel {

	private JSlider stressSlider[] = new JSlider[Globals.MAX_ENCODER_NO];
	private JPanel sPanel;

	public StressPanel() {
		// set size
		Dimension stressPanelDimension = new Dimension(Globals.leftPanelWidth, Globals.leftPanelWindowsHeight);
		setPreferredSize(stressPanelDimension);
		setMinimumSize(stressPanelDimension);
		setMaximumSize(stressPanelDimension);

		// set layout and border
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createEtchedBorder());

		// make rigid area
		add(Box.createRigidArea(new Dimension(Globals.leftPanelWidth, 20)));

		// make label
		JLabel label = new JLabel("Stress Emulation");
		label.setFont(new Font("TimesRoman", Font.ITALIC | Font.BOLD, 20));
		label.setAlignmentX(Component.CENTER_ALIGNMENT);
		add(label);

		// make rigid area
		add(Box.createRigidArea(new Dimension(Globals.leftPanelWidth, 10)));

		MakeSPanel();
		add(sPanel);
	}
	public void Show() {
		setVisible(true);
	}
	public void Hide() {
		setVisible(false);
	}

	private void MakeSPanel() {
		sPanel = new JPanel(false);
		// set size
		Dimension panelDimension = new Dimension(Globals.leftPanelWidth - 20, 200);
		sPanel.setPreferredSize(panelDimension);
		sPanel.setMinimumSize(panelDimension);
		sPanel.setMaximumSize(panelDimension);
	}
	public void UpdateStressPanel() {
		// Update stress Slider
		sPanel.removeAll();
		JPanel subPanel[] = new JPanel[Globals.encListModel.getSize()];

		for (int i=0; i < Globals.MAX_ENCODER_NO; i++) {
			stressSlider[i] = null;
		}

		Hashtable stressLabelTable = new Hashtable();
		stressLabelTable.put(new Integer(1), new JLabel("Low"));
		stressLabelTable.put(new Integer(500), new JLabel("Medium"));
		stressLabelTable.put(new Integer(1000), new JLabel("High"));

		for (int i=0; i < Globals.encListModel.getSize(); i++) {
			subPanel[i] = new JPanel();
			subPanel[i].setBorder(BorderFactory.createTitledBorder(Globals.encListModel.getElementAt(i).toString()));
			subPanel[i].setLayout(new GridLayout(1,1));
			stressSlider[i] = new JSlider(1, 1000, Globals.PFM_INIT);
			stressSlider[i].setLabelTable(stressLabelTable);
			stressSlider[i].setPaintLabels(true);
			stressSlider[i].setMajorTickSpacing(100);
			stressSlider[i].setPaintTicks(true);
			stressSlider[i].addChangeListener(new StressChange());
			subPanel[i].add(stressSlider[i]);
			sPanel.add(subPanel[i]);
		}
		sPanel.validate();
	}

	class StressChange implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider)e.getSource();


			if (!source.getValueIsAdjusting()) {
				int index = 0;
				for (int i=0; i < Globals.encListModel.getSize(); i++) {
					if (stressSlider[i] == source) {
						index = i;
					}
				}
				String hostName = Globals.encListModel.getElementAt(index).toString();
				if (hostName != "") {
					int stressValue = (int)((JSlider)e.getSource()).getValue();
					try {
						Socket stressSocket = new Socket(hostName, Globals.STRESS_PORT);
						PrintStream stressOutputStream = new PrintStream(stressSocket.getOutputStream());
						stressOutputStream.print("load "+stressValue);
						stressOutputStream.flush();
						stressSocket.close();
					} catch (UnknownHostException sE) {
						System.err.println(sE);
					} catch (IOException sE) {
						System.err.println(sE);
					}
					// Add status message
					Globals.statusTextArea.append("Stress of " + hostName + "is now " + stressValue + "\n");

					System.out.println("Load Control Value of " + hostName + " = " + stressValue);
				}
				Toolkit.getDefaultToolkit().beep();
			}
		}
	}
}
