/* patched version from July 13, 1994 */
/* mpeg2dec.c, main(), initialization, option processing                    */

/* Copyright (C) 1994, MPEG Software Simulation Group. All Rights Reserved. */

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 * The MPEG Software Simulation Group does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
 * are subject to royalty fees to patent holders.  Many of these patents are
 * general enough such that they are unavoidable regardless of implementation
 * design.
 *
 */

// MS Windows includes
#include <winsock2.h>
#include <windows.h>
#include <commdlg.h>
#include <assert.h>
#include <io.h>
#include "gui.h"
#define	APPNAME         "Mpeg2Play"	/* The name of this application */
#define BASEWINDOWTITLE "mpeg2play"	/* The title bar text */
#define MAXFILENAME     256	/* maximum length of file pathname */
#define MAXSTRLENGTH	256	/* maximum length of string */
#define MAXCUSTFILTER   40	/* maximum size of custom filter buffer */
#define CONFIGFILENAME	"PXNet.conf"	/* The name of configuration file */

// display related variables
#define START	0
#define READY   1
#define PLAYING 2
#define PAUSED  3
#define FINISH  4
#define STOP    5


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>

#define GLOBAL
#include "config.h"
#include "global.h"
#include "mpeg2dec.h"
#include "resource.h"

// MS Windows related variables
LRESULT CALLBACK WndProc(HWND hWnd,	// window handle
			 UINT message,	// type of message
			 WPARAM uParam,	// additional information
			 LPARAM lParam);	// additional information
static HINSTANCE hInst;		// current instance
static HWND ghWnd = NULL;	// handle to main window
static char *image = NULL;
static char szHostName[MAXSTRLENGTH];
static int iPortNumber;
static unsigned short gusState = START;
static int nDitherType;
static HDC hDC;
static HPALETTE hpal, hPalPrev = NULL;
static PBITMAPINFO pbmi = NULL;

HWND hwndDesktop;
RECT rcDesktop;
int startx, starty;

int play_movie(void);
extern int __argc;
extern char **__argv;

/* private prototypes */
static void initdecoder _ANSI_ARGS_((void));
static void options _ANSI_ARGS_((int *argcp, char **argvp[]));
static int getval _ANSI_ARGS_((char *argv[]));
BITMAPFILEHEADER *DibLoadImage(PTSTR pstrFileName);
void DecorateWindow();

/* loop flag not implemented yet */
static int loopflag;

extern int readCnt;
// MS Windows related functions
/////////////////////////////////////////////////////////////////////////////
/*
 *******************************************************************************
 *
 * WinMain:
 *
 *******************************************************************************
*/

//
// yield function to simulate multitasking on MS Windows
//
static void myYield(void)
{
	MSG msg;

	if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
		TranslateMessage(&msg);	// Translates virtual key codes
		DispatchMessage(&msg);	// Dispatches message to window
	}
}

//
// initializations
//
static BOOL InitApplication(HINSTANCE hInstance)
{
	WNDCLASS wc;

	// Fill in window class structure with parameters that describe the
	// main window.

	wc.style = CS_OWNDC | CS_HREDRAW | CS_VREDRAW;	// Class style(s).
	wc.lpfnWndProc = (WNDPROC) WndProc;	// Window Procedure
	wc.cbClsExtra = 0;	// No per-class extra data.
	wc.cbWndExtra = 0;	// No per-window extra data.
	wc.hInstance = hInstance;	// Owner of this class
	wc.hIcon = LoadIcon(hInstance, APPNAME);	// Icon name from .RC
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);	// Cursor
	wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);	// Default color
	wc.lpszMenuName = APPNAME;	// Menu name from .RC
	wc.lpszClassName = APPNAME;	// Name to register as

	// Register the window class and return success/failure code.
	return (RegisterClass(&wc));
}

static BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{

	// Save the instance handle in static variable, which will be used in
	// many subsequence calls from this application to Windows.

	hInst = hInstance;	// Store instance handle in our global variable

	// get desktop window information
	hwndDesktop = GetDesktopWindow();
	GetWindowRect(hwndDesktop, &rcDesktop);

	// Create a main window for this application instance.
	ghWnd = CreateWindow(APPNAME,	// See RegisterClass() call.
			     BASEWINDOWTITLE,	// Text for window title bar.
			     WS_OVERLAPPEDWINDOW & ~WS_THICKFRAME & ~WS_MAXIMIZEBOX,	// Window style.
			     0, 0, rcDesktop.right - rcDesktop.left, rcDesktop.bottom - rcDesktop.top,	// Use default positioning
			     NULL,	// Overlapped windows have no parent.
			     NULL,	// Use the window class menu.
			     hInstance,	// This instance owns this window.
			     NULL	// We don't use any data in our WM_CREATE
	    );

	// If window could not be created, return "failure"
	if (!ghWnd)
		return (FALSE);

	// Make the window visible; update its client area; and return "success"
	ShowWindow(ghWnd, nCmdShow);	// Show the window
	UpdateWindow(ghWnd);	// Sends WM_PAINT message

	return (TRUE);		// We succeeded...

}

/*
 *******************************************************************************
 *
 * Open Host & Port box
 *
 *******************************************************************************
*/

BOOL CALLBACK OpenHostPort(HWND hDlg, UINT message, WPARAM wParam,
			   LPARAM lParam)
{
	char szPortNumber[MAXSTRLENGTH];
	switch (message) {
	case WM_INITDIALOG:
		// set default text
		SetDlgItemText(hDlg, IDC_HOST, szHostName);
		SetDlgItemInt(hDlg, IDC_PORT, iPortNumber, 0);
		return (TRUE);
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
			GetDlgItemText(hDlg, IDC_HOST, szHostName,
				       MAXSTRLENGTH);
			iPortNumber =
			    GetDlgItemInt(hDlg, IDC_PORT, NULL, FALSE);
			EndDialog(hDlg, TRUE);
			return (TRUE);
			break;
		case IDCANCEL:
			EndDialog(hDlg, TRUE);
			return (TRUE);
			break;
		}
		break;
	}
	return (FALSE);		// Didn't process the message
}

/*
 *******************************************************************************
 *
 * About box
 *
 *******************************************************************************
*/

BOOL CALLBACK About(HWND hDlg,	// window handle of the dialog box
		    UINT message,	// type of message
		    WPARAM wParam,	// message-specific information
		    LPARAM lParam)
{
	switch (message) {
	case WM_COMMAND:	// message: received a command
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
			EndDialog(hDlg, TRUE);	// Exit the dialog
			return (TRUE);
		}
		break;
	}
	return (FALSE);		// Didn't process the message

	lParam;			// This will prevent 'unused formal parameter' warnings
}

//
// Main window procedure
//
LRESULT CALLBACK WndProc(HWND hWnd,	// window handle
			 UINT message,	// type of message
			 WPARAM uParam,	// additional information
			 LPARAM lParam)	// additional information
{
	static BITMAPFILEHEADER *pbmfh;
	static BITMAPINFO *pbmi;
	static BYTE *pBits;
	static int cxDib, cyDib;
	HDC hdc;
	PAINTSTRUCT ps;
	int x, y;

	switch (message) {
	case WM_CREATE:
		GetConfigInfo(hWnd);
		pbmfh = DibLoadImage(TEXT("medianet.bmp"));
		if (pbmfh == NULL) {
			MessageBox(hWnd, TEXT("Cannot load DIB file"), "XPlayer", 0);
			return 0;
		}
		pbmi = (BITMAPINFO *) (pbmfh+1);
		pBits = (BYTE *)pbmfh + pbmfh->bfOffBits;
		cxDib = pbmi->bmiHeader.biWidth;
		cyDib = abs(pbmi->bmiHeader.biHeight);
		break;
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		for (y = 0; y < rcDesktop.bottom; y += cyDib) {
			for (x = 0; x < rcDesktop.right; x += cxDib) {
				SetDIBitsToDevice(hdc,
					x,y,
					cxDib, cyDib,
					0, 0,
					0, cyDib,
					pBits,
					pbmi,
					DIB_RGB_COLORS);
			}
		}
		if (horizontal_size != 0 && vertical_size != 0) {
			DecorateWindow();
		}
		EndPaint(hWnd, &ps);
		break;
	case WM_CLOSE:
		gusState = STOP;
		myYield();
		return (DefWindowProc(hWnd, message, uParam, lParam));
		break;


	case WM_DESTROY:	// message: window being destroyed
		gusState = STOP;
		myYield();
		PostQuitMessage(0);
		break;

	case WM_COMMAND:	// message: command from application menu
		{
			int wmId, wmEvent;

			wmId = LOWORD(uParam);
			wmEvent = HIWORD(uParam);

			switch (wmId) {
			case IDM_OPEN:
				{
					char szWindowTitle[80];
					char szFileTitle[MAXFILENAME];
					FARPROC lpProcOpenHostPort;

					// if something was playing, give a chance to stop.
					if (gusState > READY) {
						gusState = STOP;
						myYield();
					}

					EnableMenuItem(GetMenu(hWnd),
						       IDM_PLAY,
						       MF_GRAYED |
						       MF_DISABLED);

					lpProcOpenHostPort =
					    MakeProcInstance((FARPROC)
							     OpenHostPort,
							     hInst);
					DialogBox(hInst, "OPEN_HOSTPORT",
						  hWnd,
						  (DLGPROC)
						  lpProcOpenHostPort);
					FreeProcInstance
					    (lpProcOpenHostPort);
					lstrcpy(szFileTitle, szHostName);


					lstrcpy(szWindowTitle,
						BASEWINDOWTITLE);
					lstrcat(szWindowTitle, " - ");
					lstrcat(szWindowTitle,
						szFileTitle);
					SetWindowText(hWnd, szFileTitle);


					// enable play menu
					EnableMenuItem(GetMenu(hWnd),
						       IDM_PLAY,
						       MF_ENABLED);

					gusState = READY;
					break;
				}

			case IDM_PLAY:
				// disable play menu while playing
				EnableMenuItem(GetMenu(hWnd), IDM_STOP,
					       MF_ENABLED);
				EnableMenuItem(GetMenu(hWnd), IDM_PLAY,
					       MF_GRAYED | MF_DISABLED);

				// play the movie. It returns after the playing ended
				/*
				   while (play_movie() >= 0) {
				   ;
				   }
				 */

				if (play_movie() < 0)
					error
					    ("error. Movie does not play.");


				// disable stop menu
				EnableMenuItem(GetMenu(hWnd), IDM_STOP,
					       MF_GRAYED | MF_DISABLED);
				EnableMenuItem(GetMenu(hWnd), IDM_PLAY,
					       MF_ENABLED);

				break;

			case IDM_STOP:
				gusState = STOP;
				// force a task switch to allow the decoder to see the stop command
				myYield();
				break;

			case IDM_EXIT:
				SendMessage(hWnd, WM_CLOSE, 0, 0l);
				break;

			case IDM_ABOUT:
				{
					FARPROC lpProcAbout;	// pointer to the "About" function

					lpProcAbout =
					    MakeProcInstance((FARPROC)
							     About, hInst);
					DialogBox(hInst,	// current instance
						  "AboutBox",	// dlg resource to use
						  hWnd,	// parent handle
						  (DLGPROC) lpProcAbout);	// About() instance address

					FreeProcInstance(lpProcAbout);
					break;
				}

			case IDM_DISPLAY8:
			case IDM_DISPLAY24:
				// reset the display.
				// It will be reinitialized the first time it will be called
				switch (outtype) {
				case T_X11:
					if (nDitherType == 8)
						CheckMenuItem(GetMenu
							      (hWnd),
							      IDM_DISPLAY8,
							      MF_UNCHECKED);
					else
						CheckMenuItem(GetMenu
							      (hWnd),
							      IDM_DISPLAY24,
							      MF_UNCHECKED);
					break;
				}
				switch (wmId) {
				case IDM_DISPLAY8:
					CheckMenuItem(GetMenu(hWnd),
						      IDM_DISPLAY8,
						      MF_CHECKED);
					outtype = T_X11;
					nDitherType = 8;
					break;

				case IDM_DISPLAY24:
					CheckMenuItem(GetMenu(hWnd),
						      IDM_DISPLAY24,
						      MF_CHECKED);
					outtype = T_X11;
					nDitherType = 24;
					break;
				}
			}
			break;
		}

	default:		// Passes it on if unproccessed
		return (DefWindowProc(hWnd, message, uParam, lParam));
	}
	return (0);
}

//
// program entry point
//
int APIENTRY WinMain(HINSTANCE hInstance,
		     HINSTANCE hPrevInstance,
		     LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;

	if (!hPrevInstance)
		// Other instances of app running?
		if (!InitApplication(hInstance))
			// Initialize shared things
			return (FALSE);	// Exits if unable to initialize

	/* Perform initializations that apply to a specific instance */
	if (!InitInstance(hInstance, nCmdShow))
		return (FALSE);


	// start-up initialization
	verbose = 0;
	outtype = T_X11;
	hDC = GetDC(ghWnd);
	nDitherType = 8;
	sflag = 0;
	ld = &base;		/* select base layer context */

/*    options(&__argc,&__argv);  */

	/* Acquire and dispatch messages until a WM_QUIT message is received. */
	while (GetMessage(&msg,	// message structure
			  NULL,	// handle of window receiving the message
			  0,	// lowest message to examine
			  0))	// highest message to examine
	{
		TranslateMessage(&msg);	// Translates virtual key codes
		DispatchMessage(&msg);	// Dispatches message to window
	}

	return (msg.wParam);	// Returns the value from PostQuitMessage

	lpCmdLine;		// This will prevent 'unused formal parameter' warnings
}

//
// display initialization
//
/* Note: this was already defined in global.h, so
   I commented it out -- Chad  */
/* extern int convmat[][4]; /* defined in display.c */

static void init_display(void)
{
	pbmi =
	    (PBITMAPINFO) malloc(sizeof(BITMAPINFOHEADER) +
				 240 * sizeof(RGBQUAD));
	pbmi->bmiHeader.biSize = (LONG) sizeof(BITMAPINFOHEADER);
	pbmi->bmiHeader.biPlanes = 1;
	pbmi->bmiHeader.biCompression = 0l;
	pbmi->bmiHeader.biSizeImage = 0l;
	pbmi->bmiHeader.biXPelsPerMeter = 0l;
	pbmi->bmiHeader.biYPelsPerMeter = 0l;
	pbmi->bmiHeader.biClrUsed = 240;
	pbmi->bmiHeader.biClrImportant = 240;
	pbmi->bmiHeader.biBitCount = nDitherType;
	pbmi->bmiHeader.biWidth = coded_picture_width;
	pbmi->bmiHeader.biHeight = coded_picture_height;
	if (pbmi->bmiHeader.biBitCount == 8) {
		// for 8 BPP, build the color palette

		LOGPALETTE *plgpl;
		short *pPalIndex;
		int crv, cbu, cgu, cgv;
		int y, u, v;
		int i;

		plgpl =
		    (LOGPALETTE *) malloc(sizeof(LOGPALETTE) +
					  240 * sizeof(PALETTEENTRY));
		plgpl->palNumEntries = 240;
		plgpl->palVersion = 0x300;
		pPalIndex = (short *) pbmi->bmiColors;

		/* matrix coefficients */
		crv = convmat[matrix_coefficients][0];
		cbu = convmat[matrix_coefficients][1];
		cgu = convmat[matrix_coefficients][2];
		cgv = convmat[matrix_coefficients][3];

		for (i = 16; i < 240; i++) {
			/* color space conversion */
			y = 16 * ((i >> 4) & 15) + 8;
			u = 32 * ((i >> 2) & 3) - 48;
			v = 32 * (i & 3) - 48;

			y = 76309 * (y - 16);	/* (255/219)*65536 */

			plgpl->palPalEntry[i].peRed =
			    clp[(y + crv * v + 32768) >> 16];
			plgpl->palPalEntry[i].peGreen =
			    clp[(y - cgu * u - cgv * v + 32768) >> 16];
			plgpl->palPalEntry[i].peBlue =
			    clp[(y + cbu * u + 32786) >> 16];
			pPalIndex[i] = i;
		}
		hpal = CreatePalette(plgpl);
		free(plgpl);
		hPalPrev = SelectPalette(hDC, hpal, FALSE);
		RealizePalette(hDC);
	}
	DecorateWindow();
}

//
// display clean-up
//
static void exit_display()
{
	if (pbmi) {
		free(pbmi);
		pbmi = NULL;
	}
	if (hPalPrev) {
		SelectPalette(hDC, hPalPrev, FALSE);
		DeleteObject(hpal);
		hPalPrev = NULL;
	}
}

//
// display the image on the MS Windows screen
//
void display_image(dithered_image)
unsigned char *dithered_image;
{
	/* display dithered image */
	SetDIBitsToDevice(hDC, startx, starty, coded_picture_width,
			  coded_picture_height, 0, 0, 0,
			  coded_picture_height, dithered_image, pbmi,
			  DIB_PAL_COLORS);
}



//
// decode a file and store and/or display it
//

int play_movie(void)
{
	int framenum, first;


	/* open MPEG input file(s) */
	readCnt = 0;
	OpenNet();
	/* Look for Systems start code */
	sysstream = 0;
	initbits();
	startcode();
	switch (showbits(32)) {
	case SEQ_START_CODE:
		break;
	case PACK_START_CODE:
		sysstream = 1;
		break;
	default:
		error("This not a valid MPEG system or video stream\n");
		break;
	}
	initbits();

	first = 1;

	do {
		/* (re)set to begin of file */

		ld = &base;
		initbits();


/*  Enhancement currently commented out
    if (twostreams)
    {
      ld = &enhan;
      lseek(enhan.infile,0l,0);
      initbits();
      ld = &base;
    }
    End of commented out section */

		framenum = 0;

		while (getheader()) {
			if (first) {

/*				RECT wr, cr;
				int d;

				// size window to film size
				SetRect(&cr, 0, 0, rcDesktop.right - rcDesktop.left,
					rcDesktop.bottom - rcDesktop.top);
				wr = cr;
				AdjustWindowRect(&wr,
						 GetWindowLong(ghWnd,
							       GWL_STYLE),
						 TRUE);
				OffsetRect(&wr, -wr.left, -wr.top);
				SetWindowPos(ghWnd, HWND_TOP, 0, 0,
					     wr.right, wr.bottom,
					     SWP_NOMOVE);

				// for QSIF movies the menu will wrap around so we need
				// to increase the height of the window
				GetClientRect(ghWnd, &wr);
				d = cr.bottom - (wr.bottom - wr.top);

				if (d > 0) {
					GetWindowRect(ghWnd, &wr);
					SetWindowPos(ghWnd, HWND_TOP, 0, 0,
						     wr.right - wr.left,
						     wr.bottom - wr.top +
						     d, SWP_NOMOVE);
				}
*/
				startx = (rcDesktop.right - rcDesktop.left - horizontal_size)/2;
				starty = (rcDesktop.bottom - rcDesktop.top - vertical_size - 80)/2;

				initdecoder();

				if (outtype == T_X11) {
					init_dither(nDitherType);
					init_display();
				}

				first = 0;
			}

			getpicture(framenum);

			if (!secondfield) {
				framenum++;
				slowdown();
			}
			// give the user a chance to perform something
			myYield();
			if (gusState == STOP)
				break;
		}

		if (framenum != 0) {
			/* put last frame */
			putlast(framenum);
		}

	}
	while (loopflag);

	CloseNet();

	if (outtype == T_X11)
		exit_display();

	return 0;
}


/////////////////////////////////////////////////////////////////////
//
// MPEG related functions
//
// (portable)
//
static void initdecoder()
{
	int i, cc, size;
	static int blk_cnt_tab[3] = { 6, 8, 12 };

	/* check scalability mode of enhancement layer */
	if (twostreams && enhan.scalable_mode != SC_SNR &&
	    !(base.scalable_mode == SC_DP && base.scalable_mode == SC_DP))
		error("unsupported scalability mode\n");

	/* clip table */
	if (!(clp = (unsigned char *) malloc(1024)))
		error("malloc failed\n");

	clp += 384;

	for (i = -384; i < 640; i++)
		clp[i] = (i < 0) ? 0 : ((i > 255) ? 255 : i);

	/* force MPEG-1 parameters */
	if (!base.mpeg2) {
		prog_seq = 1;
		prog_frame = 1;
		pict_struct = FRAME_PICTURE;
		frame_pred_dct = 1;
		chroma_format = CHROMA420;
		matrix_coefficients = 5;
	}

	/* round to nearest multiple of coded macroblocks */
	mb_width = (horizontal_size + 15) / 16;
	mb_height = (base.mpeg2
		     && !prog_seq) ? 2 * ((vertical_size + 31) / 32)
	    : (vertical_size + 15) / 16;
	coded_picture_width = 16 * mb_width;
	coded_picture_height = 16 * mb_height;

	chrom_width = (chroma_format == CHROMA444) ? coded_picture_width
	    : coded_picture_width >> 1;
	chrom_height = (chroma_format != CHROMA420) ? coded_picture_height
	    : coded_picture_height >> 1;
	blk_cnt = blk_cnt_tab[chroma_format - 1];

	for (cc = 0; cc < 3; cc++) {
		if (cc == 0)
			size = coded_picture_width * coded_picture_height;
		else
			size = chrom_width * chrom_height;

		if (!(refframe[cc] = (unsigned char *) malloc(size)))
			error("malloc failed\n");

		if (!(oldrefframe[cc] = (unsigned char *) malloc(size)))
			error("malloc failed\n");

		if (!(auxframe[cc] = (unsigned char *) malloc(size)))
			error("malloc failed\n");

		if (base.scalable_mode == SC_SPAT) {
			/* this assumes lower layer is 4:2:0 */
			if (!
			    (llframe0[cc] =
			     (unsigned char *) malloc((llw * llh) /
						      (cc ? 4 : 1))))
				error("malloc failed\n");
			if (!
			    (llframe1[cc] =
			     (unsigned char *) malloc((llw * llh) /
						      (cc ? 4 : 1))))
				error("malloc failed\n");
		}
	}

	if (base.scalable_mode == SC_SPAT) {
		if (!
		    (lltmp =
		     (short *) malloc(llw * ((llh * vn) / vm) *
				      sizeof(short))))
			error("malloc failed\n");
	}

	/* IDCT */
	init_idct();
}

void error(text)
char *text;
{
	fprintf(stderr, text);
	exit(1);
}

/* compliance warning messages to user, but don't exit() */
void warning(text)
char *text;
{
	if (!quiet) {
		fprintf(stderr, "%s\n", text);
	}
}

#ifdef TRACE
/* trace output */
void printbits(code, bits, len)
int code, bits, len;
{
	int i;
	for (i = 0; i < len; i++)
		printf("%d", (code >> (bits - 1 - i)) & 1);
}
#endif

/* option processing */
/*
static void options(argcp,argvp)
int *argcp;
char **argvp[];
{
  outtype = T_X11;

  while (*argcp>1 && (*argvp)[1][0]=='-')
  {
    while ((*argvp)[1][1])
    {
      switch (toupper((*argvp)[1][1]))
      {
      case 'V':
	verbose = getval(*argvp);
	break;
      case 'O':
	outtype = getval(*argvp);
	break;
      case 'L':
	loopflag = 1;
	break;
      case 'S':
	sflag = 1;
	break;
#ifdef TRACE
      case 'T':
	trace = 1;
	break;
#endif
      case 'Q':
	quiet = 1;
	break;
      default:
	fprintf(stderr,"undefined option -%c ignored\n",(*argvp)[1][1]);
      }

      (*argvp)[1]++;
    }

    (*argvp)++;
    (*argcp)--;
  }

  if (sflag)
  {
    /* input file for spatial prediction */
/*    llinputname = (*argvp)[1];
    (*argvp)++;
    (*argcp)--;
  }

#if 0
  if (*argcp!=2 && *argcp!=3)
  {
    printf("\n%s, %s\n",version,author);
    printf("Usage:   mpeg2play {options} input.m2v {upper.m2v}\n\
Options: -vn  verbose output (n: level)\n\
	 -on  output format (0: X11 (default), 1: none\n\
	 -l   loop sequence\n\
	 -q   quiet (no error output)\n\
	 -s infile  spatial scalable sequence\n");
    exit(0);
  }
#endif
}
*/

static int getval(argv)
char *argv[];
{
	int val;

	if (sscanf(argv[1] + 2, "%d", &val) != 1)
		return 0;

	while (isdigit(argv[1][2]))
		argv[1]++;

	return val;
}

OpenNet()
{
	WSADATA WSAData;
	SOCKET s;
	SOCKADDR_IN sin;
	PHOSTENT phostent = NULL;
	TCHAR szError[100];
	WSAEVENT hEventObject;
	int ioctl_opt = 0;

	if (WSAStartup(MAKEWORD(2, 2), &WSAData) != 0) {
		wsprintf(szError, TEXT("WSAStartup failed. Error: %d"),
			 WSAGetLastError());
		MessageBox(NULL, szError, TEXT("Error"), MB_OK);
	}
//  if (WSAIsBlocking()) {
//        MessageBox(NULL, "Blocking.", TEXT("Blocking."), MB_OK);
//  } else {
//        MessageBox(NULL, "Not Blocking.", TEXT("Not Blocking."), MB_OK);
//  }

	if ((s = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
		wsprintf(szError,
			 TEXT("Allocating socket failed. Error: %d"),
			 WSAGetLastError());
		MessageBox(NULL, szError, TEXT("Error"), MB_OK);
	}

	ZeroMemory(&sin, sizeof(SOCKADDR_IN));
	sin.sin_family = AF_INET;
	//MessageBox(NULL, szHostName, TEXT("Host Name"), MB_OK);
	if ((phostent = gethostbyname(szHostName)) == NULL) {
		wsprintf(szError,
			 TEXT("Unable to get the host name. Error: %d"),
			 WSAGetLastError());
		MessageBox(NULL, szError, TEXT("Error"), MB_OK);
	}
	memcpy((char FAR *) &(sin.sin_addr), phostent->h_addr,
	       phostent->h_length);

	sin.sin_port = htons(iPortNumber);
	if (connect(s, (PSOCKADDR) & sin, sizeof(sin)) == SOCKET_ERROR) {
		wsprintf(szError,
			 TEXT
			 ("Connecting to the server failed. Error: %d"),
			 WSAGetLastError());
		MessageBox(NULL, szError, TEXT("Error"), MB_OK);
	}
	base.infile = s;

}

CloseNet()
{
	WSACleanup();
	close(base.infile);

}

GetConfigInfo(HWND hWnd)
{
	FILE *fp;
	TCHAR szError[100];
	char buffer[MAXSTRLENGTH];
	int i, j, k;

	if ((fp = fopen(CONFIGFILENAME, "r")) < 0) {
		wsprintf(szError, "Config file open error");
		MessageBox(NULL, szError, TEXT("Error"), MB_OK);
	} else {

		// X Control Center
		ZeroMemory(buffer, MAXSTRLENGTH);
		fscanf(fp, "%s%d", buffer, &i);

		// X Video Server
		ZeroMemory(buffer, MAXSTRLENGTH);
		fscanf(fp, "%s%d", buffer, &i);

		// X Video Player
		ZeroMemory(buffer, MAXSTRLENGTH);
		fscanf(fp, "%s%d", buffer, &i);
		iPortNumber = i;

		// X encoder luncher
		ZeroMemory(buffer, MAXSTRLENGTH);
		fscanf(fp, "%s%d%d%d", buffer, &i, &j, &k);

		// X decoder
		ZeroMemory(buffer, MAXSTRLENGTH);
		fscanf(fp, "%s%d%d", buffer, &i, &j);

		// X multiplexer
		ZeroMemory(buffer, MAXSTRLENGTH);
		fscanf(fp, "%s%d%d", buffer, &i, &j);

		strcpy(szHostName, buffer);

		EnableMenuItem(GetMenu(hWnd), IDM_PLAY, MF_ENABLED);

		gusState = READY;
		close(fp);
	}

}

slowdown()
{
	FILE *f;
	int i, j;
	int delay;

	if ((f = fopen("framerate.cfg", "r")) != NULL) {
		fscanf(f, "%d", &delay);
		fclose(f);
		for (i=0; i < delay; i++) {
			j = j++;
		}
	}
}

BITMAPFILEHEADER *DibLoadImage(PTSTR pstrFileName)
{
	BOOL bSuccess;
	DWORD dwFileSize, dwHighSize, dwBytesRead;
	HANDLE hFile;
	BITMAPFILEHEADER *pbmfh;

	hFile = CreateFile(pstrFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if (hFile == INVALID_HANDLE_VALUE)
		return NULL;
	dwFileSize = GetFileSize(hFile, &dwHighSize);

	if (dwHighSize) {
		CloseHandle(hFile);
		return NULL;
	}

	pbmfh = malloc(dwFileSize);
	if (!pbmfh) {
		CloseHandle(hFile);
		return NULL;
	}
	bSuccess = ReadFile(hFile, pbmfh, dwFileSize, &dwBytesRead, NULL);
	CloseHandle(hFile);
	if (!bSuccess || (dwBytesRead != dwFileSize) || (pbmfh->bfType != *(WORD *) "BM") || (pbmfh->bfSize != dwFileSize)) {
		free(pbmfh);
		return NULL;
	}
	return pbmfh;
}

void DecorateWindow()
{
	RECT rect;
	HBRUSH hBrush;

	SetRect(&rect, startx-5, starty-5, startx + horizontal_size + 5, starty + vertical_size + 5);
	hBrush = (HBRUSH)GetStockObject(WHITE_BRUSH);
	FillRect(hDC, &rect, hBrush);

	SetRect(&rect, startx-2, starty-2, startx + horizontal_size + 2, starty + vertical_size + 2);
	hBrush = (HBRUSH)GetStockObject(BLACK_BRUSH);
	FillRect(hDC, &rect, hBrush);
}